==> itk-viewer-bootstrap-ui-0.10.0.tgz: package/src/Images/createTransferFunctionEditor.js <==
import { throttle } from '@kitware/vtk.js/macros'
import {
  TransferFunctionEditor,
  windowPointsForSort
} from 'itk-viewer-transfer-function-editor'

const PIECEWISE_UPDATE_DELAY = 100

export const getNodes = (range, points) => {
  const delta = range[1] - range[0]
  const windowedPoints = windowPointsForSort(points)
  return windowedPoints.map(([x, y]) => ({
    x: range[0] + delta * x,
    y,
    midpoint: 0.5,
    sharpness: 0
  }))
}

// grab head and tail or fallback to data range if 1 or less points
const getRange = (dataRange, nodes) =>
  nodes.length > 1 ? [nodes[0].x, nodes[nodes.length - 1].x] : dataRange

const updateContextPiecewiseFunction = (context, dataRange, points) => {
  if (!context.images.piecewiseFunctions) return // not ready yet

  const name = context.images.selectedName
  const actorContext = context.images.actorContext.get(name)
  const component = actorContext.selectedComponent
  context.service.send({
    type: 'IMAGE_PIECEWISE_FUNCTION_POINTS_CHANGED',
    data: {
      name,
      component,
      points
    }
  })

  const nodes = getNodes(dataRange, points)
  const range = getRange(dataRange, nodes)
  context.service.send({
    type: 'IMAGE_PIECEWISE_FUNCTION_CHANGED',
    data: {
      name,
      component,
      range,
      nodes
    }
  })
}

const vtkPiecewiseGaussianWidgetFacade = (tfEditor, context) => {
  const getDataRange = () => {
    const name = context.images.selectedName
    const actorContext = context.images.actorContext.get(name)
    const component = actorContext.selectedComponent
    const dataRange = actorContext.colorRangeBounds.get(component)
    return dataRange ?? [0, 255]
  }

  const update = () =>
    updateContextPiecewiseFunction(
      context,
      getDataRange(),
      tfEditor.getPoints()
    )

  const throttledUpdate = throttle(update, PIECEWISE_UPDATE_DELAY)
  tfEditor.eventTarget.addEventListener('updated', throttledUpdate)

  const getOpacityNodes = (range = getDataRange()) =>
    getNodes(range, tfEditor.getPoints())

  const getOpacityRange = (range = getDataRange()) =>
    getRange(range, getOpacityNodes(range))

  return {
    setColorTransferFunction: (tf) => {
      tfEditor.setColorTransferFunction(tf)
    },

    setPoints(points) {
      tfEditor.setPoints(points)
      updateContextPiecewiseFunction(context, getDataRange(), points)
    },

    getPoints() {
      return tfEditor.getPoints()
    },

    setRangeZoom: (newRange) => {
      tfEditor.setViewBox(...newRange)
    },

    setDataRange: () => undefined,

    getOpacityNodes,
    getOpacityRange,
    setHistogram: (h) => tfEditor.setHistogram(h),
    render: () => undefined,

    getGaussians() {
      console.warn('getGaussians not implemented, use getPoints')
      return []
    },

    setGaussians() {
      console.warn('setGaussians not implemented, use setPoints')
    }
  }
}

export const createTransferFunctionEditor = (context, mount) => {
  const editor = new TransferFunctionEditor(mount)

  return vtkPiecewiseGaussianWidgetFacade(editor, context)
}

==> itk-viewer-bootstrap-ui-0.12.1.tgz: package/src/Images/createTransferFunctionEditor.js <==
import { TransferFunctionEditor } from 'itk-viewer-transfer-function-editor'
import { throttle } from './throttle'

const PIECEWISE_UPDATE_DELAY = 100

const updateContextPiecewiseFunction = (context, points) => {
  if (!context.images.piecewiseFunctions) return // not ready yet

  const name = context.images.selectedName
  const actorContext = context.images.actorContext.get(name)
  const component = actorContext.selectedComponent
  context.service.send({
    type: 'IMAGE_PIECEWISE_FUNCTION_POINTS_CHANGED',
    data: {
      name,
      component,
      points
    }
  })
}

const vtkPiecewiseGaussianWidgetFacade = (tfEditor, context) => {
  const update = () =>
    updateContextPiecewiseFunction(context, tfEditor.getPoints())

  const throttledUpdate = throttle(update, PIECEWISE_UPDATE_DELAY)
  tfEditor.eventTarget.addEventListener('updated', throttledUpdate)

  return {
    setColorTransferFunction: (tf) => {
      tfEditor.setColorTransferFunction(tf)
    },

    setPoints(points) {
      // tfEditor.setPoints recreates them and they loose their "grabbed" state
      // so ignore events coming down triggered by user dragging points
      const currentPoints = tfEditor.getPoints()
      const arePointsModified =
        points.length !== currentPoints.length ||
        points.some(([newX, newY], idx) => {
          const [oldX, oldY] = currentPoints[idx]
          return newX !== oldX || newY !== oldY
        })
      if (arePointsModified) tfEditor.setPoints(points)
    },

    getPoints() {
      return tfEditor.getPoints()
    },

    setRangeZoom: (newRange) => {
      tfEditor.setViewBox(...newRange)
    },

    setHistogram: (h) => tfEditor.setHistogram(h),
    render: () => undefined,

    getGaussians() {
      console.warn('getGaussians not implemented, use getPoints')
      return []
    },

    setGaussians() {
      console.warn('setGaussians not implemented, use setPoints')
    }
  }
}

export const createTransferFunctionEditor = (context, mount) => {
  const editor = new TransferFunctionEditor(mount)

  return vtkPiecewiseGaussianWidgetFacade(editor, context)
}
